file=Makefile
echo "Extracting $file"
rm -f $file

file=Makefile.$osname-$arch$suffix
rm -f $file

if test -n "$GP_INSTALL_PREFIX"; then
  date=`date +%Y%m%d%H%M`; date="-t $date"
fi

case "$status" in
  alpha|beta|kernel) suf_status=".$status";;
  *) suf_status=;;
esac

tar="/bin/tar"
filename=pari-$version.$patch$suf_status
tarfile=$filename.tar
case "`$tar 2>&1`" in
  *"--help"*) fl=cfXt; inc="";;
  *) fl=FcfX; inc="-I";;
esac
dosversion=`echo $version|sed -e 's/\.//g'`
dosversion="$dosversion$patch"

# private: for automated tests in Bordeaux
bxmachine=kochab.math.u-bordeaux.fr
gzip=/usr/local/bin/gzip

cat > $file << EOT
#
# This file was created by Configure. All changes made will be lost
# next time Configure is run.
#
SHELL = $make_sh
RM = /bin/rm -f

dft target::
	@echo "Possible targets are:"
	@echo "	gp			 Compilation"
	@echo "	bench, test, test-compat Compilation and test"
	@echo "	dobench, dotest 	 Test only"
	@echo "	doc			 Documentation"
	@echo "	install			 Installation"
	@echo "	clean, cleantest         Cleaning up"
	@echo "For pari maintainers..."
	@echo "	all			Compile gp-sta gp-dyn lib-sta lib-dyn"
	@echo "	distrib			For distribution of pari/gp"
	@echo "	bigdistrib		Idem, with all directories"
	@echo "	ctags			Generate VI/VIM tags file in ./src"
	@echo "	etags			Generate Emacs tags file in ./src"

gp all ftest bench test test-compat test-graphic install clean cleantest install-bin install-lib-sta install-bin-sta dobench dotest::
	cd $objdir; \$(MAKE) \$@

doc docps gpman::
	cd doc; \$(MAKE) \$@

RCS::
	ci -f -l -r\$\${REVISION:-$rcsrevision} -m"." \`find src -name \\*.[chs] -type f -print\`

setdir::
	@dir=pari-$version.$patch$suf_status; if test -d \$\$dir; then \\
	   echo "Remove \$\$dir before building a new release"; exit 1;\\
	fi;\\
	mkdir \$\$dir; mv $tarfile \$\$dir; \\
	cd \$\$dir; tar xf $tarfile; \$(RM) $tarfile; cd ..; \\
	tar cf $tarfile \$\$dir; \\
	rm -r \$\$dir

tarclean:
	\$(RM) examples/Makefile*
	rm -f $tarfile.gz

bigdistrib: bigtar setdir
	gzip $tarfile

uu: bigdistrib
	uuencode $tarfile.gz $tarfile.gz > pari.uu

distrib: tar setdir
	gzip $tarfile

dosdistrib: docps gpman
	-zip -kr GPB_${dosversion}.ZIP GP.EXE README README.DOS doc/gp.man doc/usersch3.tex doc/gphelp examples EMX/README.DOC RSX/README.TXT
	-zip -k GPD_${dosversion}.ZIP doc/users.ps doc/refcard.ps doc/tutorial.ps
	-zip -kr GPS_${dosversion}.ZIP \`cat config/filelist.dos\`
	-zip -k GPM_${dosversion}.ZIP doc/*.tex

bigtar: tar
	$tar rf $tarfile doc/gp.1 doc/tutorial.tex doc/app* doc/users*.tex\
          doc/macrosbig.tex doc/refcard.tex doc/translations doc/*.in

tar: tarclean
	$tar $fl $tarfile $config_dir/tar_exclude $inc $config_dir/tar_include

etags:
	cd src; sh make_emacs_tags

ctags:
	src/make_vi_tags $TOP/src

benchbx: $tarfile.gz
	rcp $tarfile.gz pari@$bxmachine:PARINEW
	rsh $bxmachine -l pari "cd PARINEW; $gzip -dc $tarfile.gz|tar xf -; ./makeall $filename"
EOT

$ln_s $file Makefile
